int     bufferSize=512;
float[][] ir, ii, or, oi;
float modifier1,modifier2;

int function = 1;   // dargestellte Funktion 
int mapping = 1;

PImage inp,powerSpektrum,phase; 

//////////////////////////////////////////////////

void generateFunctionAndFFT() {

  switch (function) {
  case 1 :  // Sinusfunkion mit verschiedenem Winkel (modifier2) 
   for (int i=0; i<bufferSize; i++) 
      for (int k=0; k<bufferSize; k++) {
        float alpha = atan2(i-bufferSize/2,k-bufferSize/2);
        float len = sqrt(sq(i-bufferSize/2)+sq(k-bufferSize/2));
        ir[i][k] = sin(sin(alpha-modifier2)*modifier1*len);
        ii[i][k] = 0;
   } break;
   
  case 2 : // eine Sprungfunktion
   for (int i=0; i<bufferSize; i++) 
      for (int k=0; k<bufferSize; k++) {
        float alpha = atan2(i-bufferSize/2,k-bufferSize/2);
        float len = sqrt(sq(i-bufferSize/2)+sq(k-bufferSize/2));
        ir[i][k] = sin(sin(alpha-modifier2)*modifier1*len)>0 ? 1 : -1;
        ii[i][k] = 0;
   } break;
    
   case 3 : // eine radiale Sprungfunkion
    for (int i=0; i<bufferSize; i++) 
      for (int k=0; k<bufferSize; k++) {
        float alpha = atan2(i-bufferSize/2,k-bufferSize/2);
        float len = sqrt(sq(i-bufferSize/2)+sq(k-bufferSize/2));
        ir[i][k] = sin(modifier1*len)>0 ? 1 : -1;
        ii[i][k] = 0;
      } break;
      
     case 4 : // Rechteck
    for (int i=0; i<bufferSize; i++) 
      for (int k=0; k<bufferSize; k++) {
        ir[i][k] = (abs(i-bufferSize/2)<50*modifier1) && (abs(k-bufferSize/2)<50*modifier1) ? 1 : 0;
        ii[i][k] = 0;
      } break;
  }

   // zeige eine skalierte Version der Funktion
   for (int i=0; i<bufferSize; i++) 
    for (int k=0; k<bufferSize; k++) 
      inp.set(i,k,color(128*(ir[i][k]+1)));
  
  // die fft überschreibt die Werte, daher kopieren wir sie
  for (int i=0; i<bufferSize; i++) 
    for (int k=0; k<bufferSize; k++) {
      or[i][k] = ir[i][k];
      oi[i][k] = ii[i][k];
    } 
    
  fft_2D(1,bufferSize,or,oi);

  // wandele Array so um, dass tiefste Frequenzen in der
  // Mitte sind
  zentriereFrequenzen(or, oi); 
  
  // zeige das gefilterte Powerspektrum
  powerSpektrum = erzeugeFFTPowerSpektrum(or, oi);   
  phase = erzeugeFFTPhase(or,oi);   
}

//////////////////////////////////////////////////
//  
// zeige die Ergebnisse der Fouriertransformierte
// in logarithmisierter Form und auf zentrierte 
// Weise mit tiefen Frequenzen in der Mitte
//
//////////////////////////////////////////////////

float mapValue(float r, float i, float minVal, float maxVal) {
  float val = 0;

  switch (mapping) {
  case 1: 
    val = 255*log(sqrt(sq(r)+sq(i)) - minVal)/log(maxVal-minVal); 
    break;
  case 2: 
    val = 255*(sqrt(sq(r)+sq(i))-minVal)/(maxVal-minVal); 
    break;
  }

  return val;
}

////////////////////////////////////////////////////////

PImage erzeugeFFTPowerSpektrum(float[][] or, float[][] oi) {

  PImage outp = createImage(or.length, or.length, RGB);
  int bs = or.length;
 // int bs2 = or.length/2;

  float maxValue = 0;
  float minValue=1e6;
  for (int i=0; i<bs; i++) 
    for (int k=0; k<bs; k++) {
      maxValue = max(maxValue, sqrt(sq(or[i][k])+sq(oi[i][k])));
      minValue = min(minValue, sqrt(sq(or[i][k])+sq(oi[i][k])));
    }

  for (int i=0; i<bs; i++) 
    for (int k=0; k<bs; k++) {
      float value = mapValue(or[i][k], oi[i][k], minValue, maxValue);
      outp.set(i, k, color(value, value, value));
    }

  return outp;
}

////////////////////////////////////////////////////////

float mapPhase(float r, float i) {

  return 255*(PI+atan2(i,r))/TWO_PI;
}

////////////////////////////////////////////////////////

PImage erzeugeFFTPhase(float[][] or, float[][] oi) {
  
  PImage outp = createImage(or.length,or.length,RGB);
  int bs = or.length;
  int bs2 = or.length/2;
  
  
  for (int i=0; i<bs; i++) 
    for (int k=0; k<bs; k++) {
      float value = mapPhase(or[i][k],oi[i][k]);
         outp.set(i, k, color(value, value, value));
    }
    return outp;
}

//////////////////////////////////////////////////

void setup()
{
  size(1600, 512, JAVA2D);
  noLoop();
 
  ir = new float[bufferSize][bufferSize];
  ii = new float[bufferSize][bufferSize];
  or = new float[bufferSize][bufferSize];
  oi = new float[bufferSize][bufferSize];
  inp = createImage(bufferSize, bufferSize, RGB);
  powerSpektrum = createImage(bufferSize, bufferSize, RGB);
  phase = createImage(bufferSize, bufferSize, RGB);
  
  modifier1 = 0.23;
  modifier2 = 0.0;
  generateFunctionAndFFT();
}

//////////////////////////////////////////////////

void draw()
{
  background(255);
  image(inp, 0,0);
  image(powerSpektrum,width/2-bufferSize/2, 0);
  image(phase,width-bufferSize, 0);
}

//////////////////////////////////////////////////

void keyPressed() {
  
  switch (key) {
    case '1' : function=1; break; // sinusfunktion
    case '2' : function=2; break; // Sprungkunftion
    case '3' : function=3; break; // radiale Sinusfunktion
    case '4' : function=4; break; // radiale Sinusfunktion
    case 'y' : mapping=1; break; // logarithmisches mapping
    case 'x' : mapping=2; break; // lineares mapping
    case '+' : modifier1 *= 1.1; break; // Frequenz
    case '-' : modifier1 /= 1.1; break;
    case 'q' : modifier2 += 0.05; break;  // Winkel
    case 'w' : modifier2 -= 0.05; break;
    case 's' : save("test.png"); break;
  }
  
  generateFunctionAndFFT();
  redraw();
}            